@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 1998, 1999, 2000, 2002, 2003, 2004
+ * Copyright (c) 1997, 1998, 1999, 2000, 2002, 2003, 2004, 2006
  *	Tama Communications Corporation
  *
  * This program is free software; you can redistribute it and/or modify
@@ -41,21 +41,24 @@ const char *gozillarc = ".gozillarc";
 #ifdef __DJGPP__
 const char *dos_gozillarc = "_gozillarc";
 #endif
+STRHASH *sh;
 
+static void load_alias(void);
 static const char *alias(const char *);
 int main(int, char **);
 void getdefinitionURL(const char *, STRBUF *);
 void getURL(const char *, STRBUF *);
 int isprotocol(const char *);
-int issource(const char *);
 int convertpath(const char *, const char *, const char *, STRBUF *);
-#if !defined(_WIN32) && !defined(__DJGPP__)
-void sendbrowser(const char *, const char *);
-#endif
-
+void show_page_by_url(const char *, const char *);
 #ifndef isblank
 #define isblank(c)	((c) == ' ' || (c) == '\t')
 #endif
+
+char cwd[MAXPATHLEN+1];
+char root[MAXPATHLEN+1];
+char dbpath[MAXPATHLEN+1];
+
 int bflag;
 int pflag;
 int qflag;
@@ -81,10 +84,7 @@ help(void)
 }
 
 /*
- * alias: get alias value.
- *
- *	i)	alias_name	alias name
- *	r)			its value
+ * load_alias: load alias value.
  *
  * [$HOME/.gozillarc]
  * +-----------------------
@@ -92,15 +92,16 @@ help(void)
  * |f = file:/usr/share/xxx.html
  * |www	http://www.xxx.yyy/
  */
-static const char *
-alias(const char *alias_name)
+static void
+load_alias(void)
 {
 	FILE *ip;
 	STRBUF *sb = strbuf_open(0);
 	char *p;
-	const char *alias = NULL;
 	int flag = STRBUF_NOCRLF;
+	struct sh_entry *ent;
 
+	sh = strhash_open(10, free);
 	if (!(p = get_home_directory()))
 		goto end;
 	if (!test("r", makepath(p, gozillarc, NULL)))
@@ -129,28 +130,62 @@ alias(const char *alias_name)
 		while (*p && isalnum(*p))	/* get name */
 			p++;
 		*p++ = 0;
-		if (!strcmp(alias_name, name)) {
-			while (*p && isblank(*p))	/* skip spaces */
+		while (*p && isblank(*p))	/* skip spaces */
+			p++;
+		if (*p == '=' || *p == ':') {
+			p++;
+			while (*p && isblank(*p))/* skip spaces */
 				p++;
-			if (*p == '=' || *p == ':') {
-				p++;
-				while (*p && isblank(*p))/* skip spaces */
-					p++;
-			}
-			value = p;
-			while (*p && !isblank(*p))	/* get value */
-				p++;
-			*p = 0;
-			alias = strmake(value, "");
-			break;
 		}
+		value = p;
+		while (*p && !isblank(*p))	/* get value */
+			p++;
+		*p = 0;
+		ent = strhash_assign(sh, name, 1);
+		if (ent->value)
+			(void)free(ent->value);
+		ent->value = check_strdup(value);
 	}
 	fclose(ip);
 end:
 	strbuf_close(sb);
-	return alias;
+}
+/*
+ * alias: get alias value.
+ *
+ *	i)	alias_name	alias name
+ *	r)			its value
+ */
+static const char *
+alias(const char *alias_name)
+{
+	struct sh_entry *ent = strhash_assign(sh, alias_name, 0);
+	return ent ? ent->value : NULL;
 }
 
+/*
+ * locate_HTMLdir: locate HTML directory made by htags(1).
+ *
+ *	r)		HTML directory
+ */
+static const char *
+locate_HTMLdir(void)
+{
+	static char htmldir[MAXPATHLEN+1];
+
+	if (test("d", makepath(dbpath, "HTML", NULL)))
+		strlimcpy(htmldir, makepath(dbpath, "HTML", NULL), sizeof(htmldir));
+	else if (test("d", makepath(root, "HTML", NULL)))
+		strlimcpy(htmldir, makepath(root, "HTML", NULL), sizeof(htmldir));
+	else if (test("d", makepath(root, "html/HTML", NULL)))
+		/* Doxygen makes HTML in doxygen's html directory. */
+		strlimcpy(htmldir, makepath(root, "html/HTML", NULL), sizeof(htmldir));
+	else
+		die("hypertext not found. See htags(1).");
+	if (vflag)
+		fprintf(stdout, "HTML directory '%s'.\n", htmldir);
+	return (const char *)htmldir;
+}
 int
 main(int argc, char **argv)
 {
@@ -205,10 +240,24 @@ main(int argc, char **argv)
 	}
 	if (show_version)
 		version(progname, vflag);
+	/*
+	 * Load aliases from .gozillarc.
+	 */
+	load_alias();
+
+	/*
+	 * Decide browser.
+	 */
 	if (!browser && getenv("BROWSER"))
 		browser = getenv("BROWSER");
+	if (!browser && alias("BROWSER"))
+		browser = alias("BROWSER");
 	if (!browser)
 		browser = "mozilla";
+
+	/*
+	 * Replace alias name.
+	 */
 	if (definition == NULL) {
 		if (argc == 0)
 			usage();
@@ -224,62 +273,25 @@ main(int argc, char **argv)
 	/*
 	 * Get URL.
 	 */
-	if (definition)
-		getdefinitionURL(definition, URL);
-	else if (isprotocol(strbuf_value(arg)))
+	if (!definition && isprotocol(strbuf_value(arg))) {
 		strbuf_puts(URL, strbuf_value(arg));
-	else
-		getURL(strbuf_value(arg), URL);
+	} else {
+		getdbpath(cwd, root, dbpath, 0);
+		if (definition)
+			getdefinitionURL(definition, URL);
+		else
+			getURL(strbuf_value(arg), URL);
+	}
 	if (pflag) {
 		fprintf(stdout, "%s\n", strbuf_value(URL));
 		if (vflag)
 			fprintf(stdout, "using browser '%s'.\n", browser);
 		exit(0);
 	}
-#ifdef _WIN32
-	if (ShellExecute(NULL, NULL, browser, strbuf_value(URL), NULL, SW_SHOWNORMAL) <= (HINSTANCE)32)
-		die("Cannot load %s (error = 0x%04x).", browser, GetLastError());
-#else
-#ifndef __DJGPP__
 	/*
-	 * send a command to browser.
+	 * Show URL's page.
 	 */
-	if (locatestring(browser, "mozilla", MATCH_AT_LAST)
-		|| locatestring(browser, "netscape", MATCH_AT_LAST)
-		|| locatestring(browser, "netscape-remote", MATCH_AT_LAST))
-	{
-		sendbrowser(browser, strbuf_value(URL));
-	}
-	/*
-	 * execute generic browser.
-	 */
-	else
-#endif /* !__DJGPP__ */
-	{
-		char com[MAXFILLEN+1];
-
-#ifdef __DJGPP__
-		char *path;
-		/*
-		 * assume a Windows browser if it's not on the path.
-		 */
-		if (!(path = usable(browser)))
-		{
-			/*
-			 * START is an internal command in XP, external in 9X.
-			 */
-			if (!(path = usable("start")))
-				path = "cmd /c start";
-			snprintf(com, sizeof(com), "%s %s \"%s\"", path, browser, strbuf_value(URL));
-		}
-		else
-			snprintf(com, sizeof(com), "%s \"%s\"", path, strbuf_value(URL));
-#else
-		snprintf(com, sizeof(com), "%s \"%s\"", browser, strbuf_value(URL));
-#endif /* !__DJGPP__ */
-		system(com);
-	}
-#endif /* _WIN32 */
+	show_page_by_url(browser, strbuf_value(URL));
 	exit(0);
 }
 
@@ -292,27 +304,13 @@ main(int argc, char **argv)
 void
 getdefinitionURL(const char *arg, STRBUF *URL)
 {
-	char cwd[MAXPATHLEN+1];
-	char root[MAXPATHLEN+1];
-	char dbpath[MAXPATHLEN+1];
-	char htmldir[MAXPATHLEN+1];
 	char *path, *p;
 	STRBUF *sb = NULL;
 	DBOP *dbop = NULL;
 	SPLIT ptable;
 	int status = -1;
+	const char *htmldir = locate_HTMLdir();
 
-	/*
-	 * get current, root and dbpath directory.
-	 * if GTAGS not found, getdbpath doesn't return.
-	 */
-	getdbpath(cwd, root, dbpath, 0);
-	if (test("d", makepath(dbpath, "HTML", NULL)))
-		strlimcpy(htmldir, makepath(dbpath, "HTML", NULL), sizeof(htmldir));
-	else if (test("d", makepath(root, "HTML", NULL)))
-		strlimcpy(htmldir, makepath(root, "HTML", NULL), sizeof(htmldir));
-	else
-		die("hypertext not found. See htags(1).");
 	path = makepath(htmldir, "MAP.db", NULL);
 	if (test("f", path))
 		dbop = dbop_open(path, 0, 0, 0);
@@ -364,46 +362,30 @@ getdefinitionURL(const char *arg, STRBUF *URL)
 		strbuf_close(sb);
 }
 /*
- * getURL: get specified URL.
+ * getURL: get URL of the specified file.
  *
- *	i)	arg	definition name
+ *	i)	file	file name
  *	o)	URL	URL begin with 'file:'
  */
 void
-getURL(const char *arg, STRBUF *URL)
+getURL(const char *file, STRBUF *URL)
 {
-	char cwd[MAXPATHLEN+1];
-	char root[MAXPATHLEN+1];
-	char dbpath[MAXPATHLEN+1];
-	char htmldir[MAXPATHLEN+1];
 	char *abspath, *p;
 	char buf[MAXPATHLEN+1];
+	STRBUF *sb = strbuf_open(0);
+	const char *htmldir = locate_HTMLdir();
 
-	if (!test("f", arg) && !test("d", NULL))
-		die("path '%s' not found.", arg);
-	if (!(abspath = realpath(arg, buf)))
-		die("cannot make absolute path name. realpath(%s) failed.", arg);
+	if (!test("f", file) && !test("d", file))
+		die("path '%s' not found.", file);
+	if (!(abspath = realpath(file, buf)))
+		die("cannot make absolute path name. realpath(%s) failed.", file);
 	if (!isabspath(abspath))
 		die("realpath(3) is not compatible with BSD version.");
-	if (issource(abspath)) {
-		STRBUF *sb = strbuf_open(0);
-		/*
-		 * get current, root and dbpath directory.
-		 * if GTAGS not found, getdbpath doesn't return.
-		 */
-		getdbpath(cwd, root, dbpath, 0);
-		if (test("d", makepath(dbpath, "HTML", NULL)))
-			strlimcpy(htmldir, makepath(dbpath, "HTML", NULL), sizeof(htmldir));
-		else if (test("d", makepath(root, "HTML", NULL)))
-			strlimcpy(htmldir, makepath(root, "HTML", NULL), sizeof(htmldir));
-		else
-			die("hypertext not found. See htags(1).");
-		/*
-		 * convert path into hypertext.
-		 */
-		p = abspath + strlen(root);
-		if (convertpath(dbpath, htmldir, p, sb) == -1)
-			die("cannot find the hypertext.");
+	/*
+	 * convert path into hypertext.
+	 */
+	p = abspath + strlen(root);
+	if (convertpath(dbpath, htmldir, p, sb) == 0) {
 		p = strbuf_value(sb);
 		/*
 		 * Make URL.
@@ -456,39 +438,6 @@ isprotocol(const char *url)
 		return 1;
 	return 0;
 }
-/*
- * issource: return 1 if path is a source file.
- *
- *	i)	path	path
- *	r)		1: source file, 0: not source file
- */
-int
-issource(const char *path)
-{
-	STRBUF *sb = strbuf_open(0);
-	char *p;
-	char suff[MAXPATHLEN+1];
-	int retval = 0;
-
-	if (!getconfs("suffixes", sb)) {
-		strbuf_close(sb);
-		return 0;
-	}
-	suff[0] = '.';
-	for (p = strbuf_value(sb); p; ) {
-		char *unit = p;
-		if ((p = locatestring(p, ",", MATCH_FIRST)) != NULL)
-			*p++ = 0;
-		strlimcpy(&suff[1], unit, sizeof(suff) - 1);
-		if (locatestring(path, suff, MATCH_AT_LAST)) {
-			retval = 1;
-			break;
-		}
-	}
-	strbuf_close(sb);
-	return retval;
-
-}
 /*
  * convertpath: convert source file into hypertext path.
  *
@@ -555,29 +504,69 @@ convertpath(const char *dbpath, const char *htmldir, const char *path, STRBUF *s
 	}
 	return -1;
 }
-#if !defined(_WIN32) && !defined(__DJGPP__)
 /*
- * sendbrowser: send message to mozilla.
+ * show_page_by_url: show page by url
  *
- *	i)	browser {mozilla|netscape}
+ *	i)	browser browser name
  *	i)	url	URL
- *
  */
+#if defined(_WIN32)
+/* Windows32 version */
 void
-sendbrowser(const char *browser, const char *url)
+show_page_by_url(const char *browser, const char *url)
 {
-	int pid;
-	char com[1024], *path;
-
-	if (!(path = usable(browser)))
-		die("%s not found in your path.", browser);
-	snprintf(com, sizeof(com), "openURL(%s)", url);
-	if ((pid = fork()) < 0) {
-		die("cannot load mozilla (fork).");
-	} else if (pid == 0) {
-		(void)close(1);
-		execlp(path, browser, "-remote", com, NULL);
-		die("cannot load %s (execlp).", browser);
-	}
+	if (ShellExecute(NULL, NULL, browser, url, NULL, SW_SHOWNORMAL) <= (HINSTANCE)32)
+		die("Cannot load %s (error = 0x%04x).", browser, GetLastError());
 }
-#endif /* !_WIN32 and !__DJGPP__ */
+#elif defined(__DJGPP__)
+/* DJGPP version */
+void
+show_page_by_url(const char *browser, const char *url)
+{
+	char com[MAXFILLEN+1];
+	char *path;
+
+	/*
+	 * assume a Windows browser if it's not on the path.
+	 */
+	if (!(path = usable(browser))) {
+		/*
+		 * START is an internal command in XP, external in 9X.
+		 */
+		if (!(path = usable("start")))
+			path = "cmd /c start";
+		snprintf(com, sizeof(com), "%s %s \"%s\"", path, browser, url);
+	} else {
+		snprintf(com, sizeof(com), "%s \"%s\"", path, url);
+	}
+	system(com);
+}
+#else
+/* UNIX version */
+void
+show_page_by_url(const char *browser, const char *url)
+{
+	char com[1024];
+
+	/*
+	 * Browsers which have openURL() command.
+	 */
+	if (locatestring(browser, "mozilla", MATCH_AT_LAST) ||
+	    locatestring(browser, "netscape", MATCH_AT_LAST) ||
+	    locatestring(browser, "netscape-remote", MATCH_AT_LAST))
+	{
+		char *path;
+
+		if (!(path = usable(browser)))
+			die("%s not found in your path.", browser);
+		snprintf(com, sizeof(com), "%s -remote \"openURL(%s)\"", path, url);
+	}
+	/*
+	 * Generic browser.
+	 */
+	else {
+		snprintf(com, sizeof(com), "%s \"%s\"", browser, url);
+	}
+	system(com);
+}
+#endif
