@@ -72,8 +72,7 @@ static int namespacelevel;	/* namespace block level */
  * Cpp: read C++ file and pickup tag entries.
  */
 void
-Cpp(file)
-	const char *file;
+Cpp(const char *file)
 {
 	int c, cc;
 	int savelevel;
@@ -533,8 +532,7 @@ Cpp(file)
  *	r)	target type
  */
 static void
-process_attribute(target)
-	int target;
+process_attribute(int target)
 {
 	int brace = 0;
 	int c;
@@ -566,8 +564,7 @@ process_attribute(target)
  *	r)	target type
  */
 static int
-function_definition(target)
-	int target;
+function_definition(int target)
 {
 	int c;
 	int brace_level;
@@ -660,9 +657,7 @@ function_definition(target)
  *	i)	target	current target
  */
 static void
-condition_macro(cc, target)
-	int cc;
-	int target;
+condition_macro(int cc, int target)
 {
 	cur = &pifstack[piflevel];
 	if (cc == SHARP_IFDEF || cc == SHARP_IFNDEF || cc == SHARP_IF) {
@@ -728,8 +723,7 @@ condition_macro(cc, target)
  *	r)		0: not data type, 1: data type
  */
 static int
-seems_datatype(token)
-	const char *token;
+seems_datatype(const char *token)
 {
 	int length = strlen(token);
 	const char *p = token + length;
