@@ -44,10 +44,7 @@
  *	gi)	tag cache
  */
 int
-makedefineindex(file, total, defines)
-	const char *file;
-	int total;
-	STRBUF *defines;
+makedefineindex(const char *file, int total, STRBUF *defines)
 {
 	int count = 0;
 	int alpha_count = 0;
@@ -85,7 +82,7 @@ makedefineindex(file, total, defines)
 	fputs_nl(header_end, DEFINES);
 	if (!aflag && !Fflag) {
 		fputs(gen_href_begin_with_title(NULL, indexlink, normal_suffix, NULL, index_string), DEFINES);
-		if (icon_list)
+		if (Iflag)
 			fputs(gen_image(CURRENT, back_icon, ".."), DEFINES);
 		else
 			fputs("[..]", DEFINES);
@@ -99,7 +96,7 @@ makedefineindex(file, total, defines)
 	 * map DEFINES to STDOUT.
 	 */
 	STDOUT = DEFINES;
-	snprintf(command, sizeof(command), "global -c");
+	snprintf(command, sizeof(command), "%s -c", global_path);
 	if ((TAGS = popen(command, "r")) == NULL)
 		die("cannot fork.");
 	alpha[0] = '\0';
@@ -110,8 +107,8 @@ makedefineindex(file, total, defines)
 		count++;
 		tag = _;
 		message(" [%d/%d] adding %s", count, total, tag);
-		if (aflag && (alpha[0] == '\0' || strncmp(tag, alpha, strlen(alpha)))) {
-			const char *msg = (alpha_count == 1) ? "definition is contained." : "definitions are contained.";
+		if (aflag && (alpha[0] == '\0' || !locatestring(tag, alpha, MATCH_AT_FIRST))) {
+			const char *msg = (alpha_count == 1) ? "definition" : "definitions";
 			int c;
 
 			if (alpha[0]) {
@@ -126,7 +123,7 @@ makedefineindex(file, total, defines)
 				else
 					fputs_nl(br, ALPHA);
 				fputs(gen_href_begin_with_title(NULL, indexlink, normal_suffix, NULL, index_string), ALPHA);
-				if (icon_list)
+				if (Iflag)
 					fputs(gen_image(PARENT, back_icon, ".."), ALPHA);
 				else
 					fputs("[..]", ALPHA);
@@ -138,7 +135,7 @@ makedefineindex(file, total, defines)
 						die("terminated abnormally.");
 				} else
 					fclose(ALPHA);
-				file_count++;
+				html_count++;
 			}
 			/*
 			 * setup index char (for example, 'a' of '[a]').
@@ -188,7 +185,7 @@ makedefineindex(file, total, defines)
 			fputs_nl(body_begin, ALPHA);
 			fprintf(ALPHA, "%s[%s]%s\n", header_begin, alpha, header_end);
 			fputs(gen_href_begin_with_title(NULL, indexlink, normal_suffix, NULL, index_string), ALPHA);
-			if (icon_list)
+			if (Iflag)
 				fputs(gen_image(PARENT, back_icon, ".."), ALPHA);
 			else
 				fputs("[..]", ALPHA);
@@ -239,7 +236,7 @@ makedefineindex(file, total, defines)
 			SPLIT ptable;
 			const char *lno, *fid, *path;
 
-			if (split((char *)line, 3, &ptable) < 3) {
+			if (split((char *)line, 2, &ptable) < 2) {
 				recover(&ptable);
 				die("too small number of parts in makedefineindex().\n'%s'", line);
 			}
@@ -273,7 +270,7 @@ makedefineindex(file, total, defines)
 		die("'%s' failed.", command);
 	if (aflag && alpha[0]) {
 		char tmp[128];
-		const char *msg = (alpha_count == 1) ? "definition is contained." : "definitions are contained.";
+		const char *msg = (alpha_count == 1) ? "definition" : "definitions";
 
 		snprintf(tmp, sizeof(tmp), "%d %s", alpha_count, msg);
 		strbuf_puts(defines, gen_href_begin_with_title("defines", alpha_f, HTML, NULL, tmp));
@@ -284,7 +281,7 @@ makedefineindex(file, total, defines)
 		else
 			fputs_nl(br, ALPHA);
 		fputs(gen_href_begin_with_title(NULL, indexlink, normal_suffix, NULL, index_string), ALPHA);
-		if (icon_list)
+		if (Iflag)
 			fputs(gen_image(PARENT, back_icon, ".."), ALPHA);
 		else
 			fputs("[..]", ALPHA);
@@ -292,7 +289,7 @@ makedefineindex(file, total, defines)
 		fputs_nl(body_end, ALPHA);
 		fputs_nl(gen_page_end(), ALPHA);
 		fclose(ALPHA);
-		file_count++;
+		html_count++;
 
 		fputs(strbuf_value(defines), DEFINES);
 	}
@@ -300,7 +297,7 @@ makedefineindex(file, total, defines)
 		fputs_nl(list_end, DEFINES);
 	if (!aflag && !Fflag) {
 		fputs(gen_href_begin_with_title(NULL, "mains", normal_suffix, NULL, index_string), DEFINES);
-		if (icon_list)
+		if (Iflag)
 			fputs(gen_image(CURRENT, back_icon, ".."), DEFINES);
 		else
 			fputs("[..]", DEFINES);
@@ -309,7 +306,7 @@ makedefineindex(file, total, defines)
 	fputs_nl(body_end, DEFINES);
 	fputs_nl(gen_page_end(), DEFINES);
 	fclose(DEFINES);
-	file_count++;
+	html_count++;
 	if (map_file)
 		fclose(MAP);
 	strbuf_close(sb);
