@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005
- *	Tama Communications Corporation
+ * Copyright (c) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+ *      2006 Tama Communications Corporation
  *
  * This file is part of GNU GLOBAL.
  *
@@ -40,6 +40,7 @@
 #include <sys/param.h>
 #include <time.h>
 
+#include "checkalloc.h"
 #include "getopt.h"
 #include "regex.h"
 #include "global.h"
@@ -67,7 +68,7 @@ void makeincludeindex();
  */
 int w32 = W32;				/* Windows32 environment	*/
 const char *www = "http://www.gnu.org/software/global/";
-int file_count = 0;
+int html_count = 0;
 int sep = '/';
 const char *save_config;
 const char *save_argv;
@@ -91,8 +92,9 @@ int aflag;				/* --alphabet(-a) option	*/
 int cflag;				/* --compact(-c) option		*/
 int fflag;				/* --form(-f) option		*/
 int Fflag;				/* --frame(-F) option		*/
-int nflag;				/* --line-number(-n) option	*/
 int gflag;				/* --gtags(-g) option		*/
+int Iflag;				/* --icon(-I) option		*/
+int nflag;				/* --line-number(-n) option	*/
 int Sflag;				/* --secure-cgi(-S) option	*/
 int qflag;
 int vflag;				/* --verbose(-v) option		*/
@@ -110,7 +112,7 @@ int copy_files;				/* 1: copy tag files		*/
 int no_map_file;			/* 1: doesn't make map file	*/
 int no_order_list;			/* 1: doesn't use order list	*/
 int other_files;			/* 1: list other files		*/
-int enable_grep;			/* 1: enable grep		*/
+int enable_grep = 1;			/* 1: enable grep		*/
 int enable_idutils;			/* 1: enable idutils		*/
 int enable_xhtml;			/* 1: enable XHTML		*/
 
@@ -180,20 +182,42 @@ const char *dir_icon  = "dir";
 const char *c_icon = "c";
 const char *file_icon = "text";
 
+const char *icon_files[] = {
+	"first",
+	"last",
+	"left",
+	"right",
+	"top",
+	"bottom",
+	"n_first",
+	"n_last",
+	"n_left",
+	"n_right",
+	"n_top",
+	"n_bottom",
+	"index",
+	"help",
+	"back",
+	"dir",
+	"c",
+	"text",
+	"pglobe"
+};
 /*
  * Configuration parameters.
  */
 int ncol = 4;				/* columns of line number	*/
 int tabs = 8;				/* tab skip			*/
+int flist_fields = 5;			/* fields number of file list	*/
 int full_path = 0;			/* file index format		*/
 int map_file = 1;			/* 1: create MAP file		*/
-const char *icon_list = NULL;		/* use icon list		*/
 const char *icon_suffix = "png";	/* icon suffix (jpg, png etc)	*/
 const char *icon_spec = "border='0' align='top'";/* parameter in IMG tag*/
 const char *prolog_script = NULL;	/* include script at first	*/
 const char *epilog_script = NULL;	/* include script at last	*/
 int show_position = 0;			/* show current position	*/
 int table_list = 0;			/* tag list using table tag	*/
+int table_flist = 0;			/* file list using table tag	*/
 int colorize_warned_line = 0;		/* colorize warned line		*/
 const char *script_alias = "/cgi-bin";	/* script alias of WWW server	*/
 const char *gzipped_suffix = "ghtml";	/* suffix of gzipped html file	*/
@@ -216,11 +240,14 @@ static struct option const long_options[] = {
         {"form", no_argument, NULL, 'f'},
         {"frame", no_argument, NULL, 'F'},
         {"gtags", no_argument, NULL, 'g'},
+        {"func-header", no_argument, NULL, 'h'},
+        {"icon", no_argument, NULL, 'I'},
         {"main-func", required_argument, NULL, 'm'},
         {"line-number", no_argument, NULL, 'n'},
         {"other", no_argument, NULL, 'o'},
         {"symbol", no_argument, NULL, 's'},
         {"secure-cgi", required_argument, NULL, 'S'},
+        {"table-flist", optional_argument, NULL, 'T'},
         {"title", required_argument, NULL, 't'},
         {"verbose", no_argument, NULL, 'v'},
         {"warning", no_argument, NULL, 'w'},
@@ -298,6 +325,18 @@ signal_setup(void)
 #endif
 }
 
+/*
+ * make directory in the dist directory.
+ */
+static void
+make_directory_in_distpath(const char *name)
+{
+	const char *path = makepath(distpath, name, NULL);
+
+	if (!test("d", path))
+		if (mkdir(path, 0775))
+			die("cannot make directory '%s'.", path);
+}
 /*
  * generate_file: generate file with replacing macro.
  *
@@ -409,7 +448,7 @@ generate_file(const char *dist, const char *file)
 	fclose(ip);
 	strbuf_close(sb);
 	regfree(&preg);
-	file_count++;
+	html_count++;
 }
 
 /*
@@ -478,7 +517,7 @@ makerebuild(const char *file)
 static void
 makehelp(const char *file)
 {
-	const char **label = icon_list ? anchor_comment : anchor_label;
+	const char **label = Iflag ? anchor_comment : anchor_label;
 	const char **icons = anchor_icons;
 	const char **msg   = anchor_msg;
 	int n, last = 7;
@@ -492,11 +531,11 @@ makehelp(const char *file)
 	fputs(header_begin, op);
 	fputs("Usage of Links", op);
 	fputs_nl(header_end, op);
-	if (!icon_list)
+	if (!Iflag)
 		fputs(verbatim_begin, op);
 	fputs("/* ", op);
 	for (n = 0; n <= last; n++) {
-		if (icon_list) {
+		if (Iflag) {
 			fputs(gen_image(CURRENT, icons[n], label[n]), op);
 			if (n < last)
 				fputc(' ', op);
@@ -507,14 +546,14 @@ makehelp(const char *file)
 	if (show_position)
 		fprintf(op, "[+line file]");
 	fputs(" */", op);
-	if (!icon_list)
+	if (!Iflag)
 		fputs_nl(verbatim_end, op);
 	else
 		fputc('\n', op);
 	fputs_nl(define_list_begin, op);
 	for (n = 0; n <= last; n++) {
 		fputs(define_term_begin, op);
-		if (icon_list) {
+		if (Iflag) {
 			fputs(gen_image(CURRENT, icons[n], label[n]), op);
 		} else {
 			fprintf(op, "[%s]", label[n]);
@@ -536,7 +575,7 @@ makehelp(const char *file)
 	fputs_nl(body_end, op);
 	fputs_nl(gen_page_end(), op);
 	fclose(op);
-	file_count++;
+	html_count++;
 }
 /*
  * makesearchpart: make search part
@@ -581,7 +620,6 @@ makesearchpart(const char *action, const char *id, const char *target)
 	}
 	strbuf_puts(sb, gen_input_radio("type", "path", 0, "Look for path name which matches to the specified pattern."));
 	strbuf_puts(sb, target ? "Path" : "Path name");
-	strbuf_puts_nl(sb, br);
 	if (enable_grep) {
 		strbuf_puts(sb, gen_input_radio("type", "grep", 0, "Retrieve lines which matches to the specified pattern."));
 		strbuf_puts_nl(sb, target ? "Grep" : "Grep pattern");
@@ -590,12 +628,20 @@ makesearchpart(const char *action, const char *id, const char *target)
 		strbuf_puts(sb, gen_input_radio("type", "idutils", 0, "Retrieve lines which matches to the specified pattern using idutils(1)."));
 		strbuf_puts_nl(sb, target ? "Id" : "Id pattern");
 	}
+	strbuf_puts_nl(sb, br);
 	strbuf_puts(sb, gen_input_checkbox("icase", "1", "Ignore case distinctions in the pattern."));
 	strbuf_puts_nl(sb, target ? "Icase" : "Ignore case");
 	if (other_files) {
 		strbuf_puts(sb, gen_input_checkbox("other", "1", "Files other than the source code are also retrieved."));
 		strbuf_puts_nl(sb, target ? "Other" : "Other files");
 	}
+	if (other_files && !target) {
+		strbuf_puts_nl(sb, br);
+		strbuf_puts(sb, "('Other files' is effective only to 'Path name'");
+		if (enable_grep)
+			strbuf_puts(sb, "and 'Grep pattern'");
+		strbuf_puts_nl(sb, ".)");
+	}
 	strbuf_puts_nl(sb, gen_form_end());
 	return strbuf_value(sb);
 }
@@ -650,7 +696,7 @@ makeindex(const char *file, const char *title, const char *index)
 		fputs_nl(gen_page_end(), op);
 	}
 	fclose(op);
-	file_count++;
+	html_count++;
 }
 /*
  * makemainindex: make main index
@@ -676,7 +722,7 @@ makemainindex(const char *file, const char *index)
 	fputs_nl(body_end, op);
 	fputs_nl(gen_page_end(), op);
 	fclose(op);
-	file_count++;
+	html_count++;
 }
 /*
  * makesearchindex: make search html
@@ -697,7 +743,7 @@ makesearchindex(const char *file)
 	fputs_nl(body_end, op);
 	fputs_nl(gen_page_end(), op);
 	fclose(op);
-	file_count++;
+	html_count++;
 }
 /*
  * makehtaccess: make .htaccess skeleton file.
@@ -815,7 +861,8 @@ makecommonpart(const char *title, const char *defines, const char *files)
 	FILE *ip;
 	STRBUF *sb = strbuf_open(0);
 	STRBUF *ib = strbuf_open(0);
-	char command[MAXFILLEN];
+	char buf[MAXFILLEN];
+	const char *tips = "Go to the GLOBAL project page.";
 	const char *_;
 
 	strbuf_puts(sb, title_begin);
@@ -823,17 +870,25 @@ makecommonpart(const char *title, const char *defines, const char *files)
 	strbuf_puts_nl(sb, title_end);
 	strbuf_puts_nl(sb, gen_div_begin("right"));
 	strbuf_sprintf(sb, "Last updated %s%s\n", now(), br);
-	strbuf_sprintf(sb, "This hypertext was generated by %sGLOBAL-%s%s.%s\n",
-		gen_href_begin_with_title_target(NULL, www, NULL, NULL, "Go to the GLOBAL project page.", "_top"),
-		get_version(),
-		gen_href_end(),
-		br);
+	if (Iflag) {
+		snprintf(buf, sizeof(buf), "Powered by GLOBAL-%s.", get_version());
+		strbuf_puts(sb, gen_href_begin_with_title_target(NULL, www, NULL, NULL, tips,"_top"));
+		strbuf_puts(sb, gen_image(CURRENT, "pglobe", buf));
+		strbuf_puts(sb, gen_href_end());
+		strbuf_puts(sb, br);
+	} else {
+		strbuf_sprintf(sb, "Powered by %sGLOBAL-%s%s.%s\n",
+			gen_href_begin_with_title_target(NULL, www, NULL, NULL, tips, "_top"),
+			get_version(),
+			gen_href_end(),
+			br);
+	}
 	strbuf_puts_nl(sb, gen_div_end());
 	strbuf_puts_nl(sb, hr);
 	if (caution) {
 		strbuf_puts_nl(sb, caution_begin);
 		strbuf_sprintf(sb, "<font size='+2' color='red'>CAUTION</font>%s\n", br);
-		strbuf_sprintf(sb, "This hypertext consist of %d files.\n", file_count);
+		strbuf_sprintf(sb, "This hypertext consist of %d files.\n", html_count);
 		strbuf_puts_nl(sb, "Please don't download whole hypertext using hypertext copy tools.");
 		strbuf_puts_nl(sb, "Our network cannot afford such traffic.");
 		strbuf_puts_nl(sb, "Instead, you can generate same thing in your computer using");
@@ -850,17 +905,17 @@ makecommonpart(const char *title, const char *defines, const char *files)
 	}
 	strbuf_sprintf(sb, "%sMAINS%s\n", header_begin, header_end);
 
-	snprintf(command, sizeof(command), "%s -x --nofilter=path %s", global_path, main_func);
-        ip = popen(command, "r");
+	snprintf(buf, sizeof(buf), "%s -x --nofilter=path %s", global_path, main_func);
+        ip = popen(buf, "r");
         if (!ip)
-                die("cannot execute command '%s'.", command);
+                die("cannot execute command '%s'.", buf);
 	strbuf_puts_nl(sb, gen_list_begin());
 	while ((_ = strbuf_fgets(ib, ip, STRBUF_NOCRLF)) != NULL) {
 		strbuf_puts_nl(sb, gen_list_body(SRCS, _));
 	}
 	strbuf_puts_nl(sb, gen_list_end());
 	if (pclose(ip) != 0)
-		die("cannot execute command '%s'.", command);
+		die("cannot execute command '%s'.", buf);
 	strbuf_puts_nl(sb, hr);
 	if (aflag && !Fflag) {
 		strbuf_puts(sb, header_begin);
@@ -885,14 +940,17 @@ makecommonpart(const char *title, const char *defines, const char *files)
 		strbuf_puts(sb, header_begin);
 		strbuf_puts(sb, title_file_index);
 		strbuf_puts_nl(sb, header_end);
-		if (!no_order_list)
+		if (table_flist)
+			strbuf_puts_nl(sb, flist_begin);
+		else if (!no_order_list)
 			strbuf_puts_nl(sb, list_begin);
 		strbuf_puts(sb, files);
-		if (!no_order_list) {
+		if (table_flist)
+			strbuf_puts_nl(sb, flist_end);
+		else if (!no_order_list)
 			strbuf_puts_nl(sb, list_end);
-		} else {
+		else
 			strbuf_puts_nl(sb, br);
-		}
 		strbuf_puts_nl(sb, hr);
 	}
 	strbuf_close(ib);
@@ -997,20 +1055,18 @@ configuration(int argc, char **argv)
 		else
 			tabs = n;
 	}
-	strbuf_reset(sb);
-	if (getconfs("gzipped_suffix", sb)) {
-		p = strdup(strbuf_value(sb));
-		if (p == NULL)
-			die("short of memory.");
-		gzipped_suffix = p;
+	if (getconfn("flist_fields", &n)) {
+		if (n < 1)
+			warning("parameter 'flist_fields' ignored becase the value (=%d) is too large or too small.", n);
+		else
+			flist_fields = n;
 	}
 	strbuf_reset(sb);
-	if (getconfs("normal_suffix", sb)) {
-		p = strdup(strbuf_value(sb));
-		if (p == NULL)
-			die("short of memory.");
-		normal_suffix = p;
-	}
+	if (getconfs("gzipped_suffix", sb))
+		gzipped_suffix = check_strdup(strbuf_value(sb));
+	strbuf_reset(sb);
+	if (getconfs("normal_suffix", sb))
+		normal_suffix = check_strdup(strbuf_value(sb));
 	strbuf_reset(sb);
 	if (getconfs("definition_header", sb)) {
 		p = strbuf_value(sb);
@@ -1025,14 +1081,16 @@ configuration(int argc, char **argv)
 	}
 	if (getconfb("other_files"))
 		other_files = 1;
-	if (getconfb("enable_grep"))
-		enable_grep = 1;
+	if (getconfb("disable_grep"))
+		enable_grep = 0;
 	if (getconfb("enable_idutils"))
 		enable_idutils = 1;
 	if (getconfb("full_path"))
 		full_path = 1;
 	if (getconfb("table_list"))
 		table_list = 1;
+	if (getconfb("table_flist"))
+		table_flist = 1;
 	if (getconfb("no_order_list"))
 		no_order_list = 1;
 	if (getconfb("copy_files"))
@@ -1040,49 +1098,24 @@ configuration(int argc, char **argv)
 	if (getconfb("no_map_file"))
 		no_map_file = 1;
 	strbuf_reset(sb);
-	if (getconfs("icon_list", sb)) {
-		p = strdup(strbuf_value(sb));
-		if (p == NULL)
-			die("short of memory.");
-		icon_list = p;
-	}
+	if (getconfs("icon_spec", sb))
+		icon_spec = check_strdup(strbuf_value(sb));
 	strbuf_reset(sb);
-	if (getconfs("icon_spec", sb)) {
-		p = strdup(strbuf_value(sb));
-		if (p == NULL)
-			die("short of memory.");
-		icon_spec = p;
-	}
+	if (getconfs("icon_suffix", sb))
+		icon_suffix = check_strdup(strbuf_value(sb));
 	strbuf_reset(sb);
-	if (getconfs("icon_suffix", sb)) {
-		p = strdup(strbuf_value(sb));
-		if (p == NULL)
-			die("short of memory.");
-		icon_suffix = p;
-	}
+	if (getconfs("prolog_script", sb))
+		prolog_script = check_strdup(strbuf_value(sb));
 	strbuf_reset(sb);
-	if (getconfs("prolog_script", sb)) {
-		p = strdup(strbuf_value(sb));
-		if (p == NULL)
-			die("short of memory.");
-		prolog_script = p;
-	}
-	strbuf_reset(sb);
-	if (getconfs("epilog_script", sb)) {
-		p = strdup(strbuf_value(sb));
-		if (p == NULL)
-			die("short of memory.");
-		epilog_script = p;
-	}
+	if (getconfs("epilog_script", sb))
+		epilog_script = check_strdup(strbuf_value(sb));
 	if (getconfb("show_position"))
 		show_position = 1;
 	if (getconfb("colorize_warned_line"))
 		colorize_warned_line = 1;
 	strbuf_reset(sb);
 	if (getconfs("script_alias", sb)) {
-		p = strdup(strbuf_value(sb));
-		if (p == NULL)
-			die("short of memory.");
+		p = check_strdup(strbuf_value(sb));
 		/* remove the last '/' */
 		q = p + strlen(p) - 1;
 		if (*q == '/')
@@ -1093,14 +1126,10 @@ configuration(int argc, char **argv)
 		dynamic = 1;
 	strbuf_reset(sb);
 	if (getconfs("body_begin", sb)) {
-		p = strdup(strbuf_value(sb));
-		if (p == NULL)
-			die("short of memory.");
+		p = check_strdup(strbuf_value(sb));
 		strbuf_reset(sb);
 		if (getconfs("body_end", sb)) {
-			q = strdup(strbuf_value(sb));
-			if (q == NULL)
-				die("short of memory.");
+			q = check_strdup(strbuf_value(sb));
 			body_begin = p;
 			body_end = q;
 		} else {
@@ -1109,14 +1138,10 @@ configuration(int argc, char **argv)
 	}
 	strbuf_reset(sb);
 	if (getconfs("table_begin", sb)) {
-		p = strdup(strbuf_value(sb));
-		if (p == NULL)
-			die("short of memory.");
+		p = check_strdup(strbuf_value(sb));
 		strbuf_reset(sb);
 		if (getconfs("table_end", sb)) {
-			q = strdup(strbuf_value(sb));
-			if (q == NULL)
-				die("short of memory.");
+			q = check_strdup(strbuf_value(sb));
 			table_begin = p;
 			table_end = q;
 		} else {
@@ -1125,14 +1150,10 @@ configuration(int argc, char **argv)
 	}
 	strbuf_reset(sb);
 	if (getconfs("title_begin", sb)) {
-		p = strdup(strbuf_value(sb));
-		if (p == NULL)
-			die("short of memory.");
+		p = check_strdup(strbuf_value(sb));
 		strbuf_reset(sb);
 		if (getconfs("title_end", sb)) {
-			q = strdup(strbuf_value(sb));
-			if (q == NULL)
-				die("short of memory.");
+			q = check_strdup(strbuf_value(sb));
 			title_begin = p;
 			title_end = q;
 		} else {
@@ -1141,14 +1162,10 @@ configuration(int argc, char **argv)
 	}
 	strbuf_reset(sb);
 	if (getconfs("comment_begin", sb)) {
-		p = strdup(strbuf_value(sb));
-		if (p == NULL)
-			die("short of memory.");
+		p = check_strdup(strbuf_value(sb));
 		strbuf_reset(sb);
 		if (getconfs("comment_end", sb)) {
-			q = strdup(strbuf_value(sb));
-			if (q == NULL)
-				die("short of memory.");
+			q = check_strdup(strbuf_value(sb));
 			comment_begin = p;
 			comment_end = q;
 		} else {
@@ -1157,14 +1174,10 @@ configuration(int argc, char **argv)
 	}
 	strbuf_reset(sb);
 	if (getconfs("sharp_begin", sb)) {
-		p = strdup(strbuf_value(sb));
-		if (p == NULL)
-			die("short of memory.");
+		p = check_strdup(strbuf_value(sb));
 		strbuf_reset(sb);
 		if (getconfs("sharp_end", sb)) {
-			q = strdup(strbuf_value(sb));
-			if (q == NULL)
-				die("short of memory.");
+			q = check_strdup(strbuf_value(sb));
 			sharp_begin = p;
 			sharp_end = q;
 		} else {
@@ -1173,14 +1186,10 @@ configuration(int argc, char **argv)
 	}
 	strbuf_reset(sb);
 	if (getconfs("brace_begin", sb)) {
-		p = strdup(strbuf_value(sb));
-		if (p == NULL)
-			die("short of memory.");
+		p = check_strdup(strbuf_value(sb));
 		strbuf_reset(sb);
 		if (getconfs("brace_end", sb)) {
-			q = strdup(strbuf_value(sb));
-			if (q == NULL)
-				die("short of memory.");
+			q = check_strdup(strbuf_value(sb));
 			brace_begin = p;
 			brace_end = q;
 		} else {
@@ -1189,14 +1198,10 @@ configuration(int argc, char **argv)
 	}
 	strbuf_reset(sb);
 	if (getconfs("reserved_begin", sb)) {
-		p = strdup(strbuf_value(sb));
-		if (p == NULL)
-			die("short of memory.");
+		p = check_strdup(strbuf_value(sb));
 		strbuf_reset(sb);
 		if (getconfs("reserved_end", sb)) {
-			q = strdup(strbuf_value(sb));
-			if (q == NULL)
-				die("short of memory.");
+			q = check_strdup(strbuf_value(sb));
 			reserved_begin = p;
 			reserved_end = q;
 		} else {
@@ -1205,14 +1210,10 @@ configuration(int argc, char **argv)
 	}
 	strbuf_reset(sb);
 	if (getconfs("position_begin", sb)) {
-		p = strdup(strbuf_value(sb));
-		if (p == NULL)
-			die("short of memory.");
+		p = check_strdup(strbuf_value(sb));
 		strbuf_reset(sb);
 		if (getconfs("position_end", sb)) {
-			q = strdup(strbuf_value(sb));
-			if (q == NULL)
-				die("short of memory.");
+			q = check_strdup(strbuf_value(sb));
 			position_begin = p;
 			position_end = q;
 		} else {
@@ -1221,14 +1222,10 @@ configuration(int argc, char **argv)
 	}
 	strbuf_reset(sb);
 	if (getconfs("warned_line_begin", sb)) {
-		p = strdup(strbuf_value(sb));
-		if (p == NULL)
-			die("short of memory.");
+		p = check_strdup(strbuf_value(sb));
 		strbuf_reset(sb);
 		if (getconfs("warned_line_end", sb)) {
-			q = strdup(strbuf_value(sb));
-			if (q == NULL)
-				die("short of memory.");
+			q = check_strdup(strbuf_value(sb));
 			warned_line_begin = p;
 			warned_line_end = q;
 		} else {
@@ -1236,34 +1233,18 @@ configuration(int argc, char **argv)
 		}
 	}
 	strbuf_reset(sb);
-	if (getconfs("include_file_suffixes", sb)) {
-		p = strdup(strbuf_value(sb));
-		if (p == NULL)
-			die("short of memory.");
-		include_file_suffixes = p;
-	}
+	if (getconfs("include_file_suffixes", sb))
+		include_file_suffixes = check_strdup(strbuf_value(sb));
 	strbuf_reset(sb);
-	if (getconfs("langmap", sb)) {
-		p = strdup(strbuf_value(sb));
-		if (p == NULL)
-			die("short of memory.");
-		langmap = p;
-	}
+	if (getconfs("langmap", sb))
+		langmap = check_strdup(strbuf_value(sb));
 	strbuf_reset(sb);
-	if (getconfs("xhtml_version", sb)) {
-		p = strdup(strbuf_value(sb));
-		if (p == NULL)
-			die("short of memory.");
-		xhtml_version = p;
-	}
+	if (getconfs("xhtml_version", sb))
+		xhtml_version = check_strdup(strbuf_value(sb));
 	/* insert htags_options into the head of ARGSV array. */
 	strbuf_reset(sb);
-	if (getconfs("htags_options", sb)) {
-		p = strdup(strbuf_value(sb));
-		if (p == NULL)
-			die("short of memory.");
-		htags_options = p;
-	}
+	if (getconfs("htags_options", sb))
+		htags_options = check_strdup(strbuf_value(sb));
 	strbuf_close(sb);
 }
 /*
@@ -1342,7 +1323,7 @@ append_options(int *argc, char **argv)
 {
 
 	STRBUF *sb = strbuf_open(0);
-	const char *p, *opt = strdup(htags_options);
+	const char *p, *opt = check_strdup(htags_options);
 	int count = 1;
 	int quote = 0;
 	const char **newargv;
@@ -1374,9 +1355,7 @@ append_options(int *argc, char **argv)
 			strbuf_putc(sb, *p);
 		}
 	}
-	newargv = (const char **)malloc(sizeof(char *) * (*argc + count + 1));
-	if (!newargv)
-		die("Short of memory.");
+	newargv = (const char **)check_malloc(sizeof(char *) * (*argc + count + 1));
 	newargv[i++] = argv[0];
 	p = strbuf_value(sb);
 	while (count--) {
@@ -1399,7 +1378,7 @@ append_options(int *argc, char **argv)
 int
 main(int argc, char **argv)
 {
-	const char *path, *av = NULL;
+	const char *av = NULL;
 	int func_total, file_total;
         char arg_dbpath[MAXPATHLEN];
 	const char *index = NULL;
@@ -1416,12 +1395,12 @@ main(int argc, char **argv)
 	save_environment(argc, argv);
 
 	/*
- 	 * insert htags_options at the head of argv.
+	 * insert htags_options at the head of argv.
 	 */
 	if (htags_options)
 		argv = append_options(&argc, argv);
 
-	while ((optchar = getopt_long(argc, argv, "acd:DfFgm:noqsS:t:vwx", long_options, &option_index)) != EOF) {
+	while ((optchar = getopt_long(argc, argv, "acd:DfFghIm:noqsS:t:Tvwx", long_options, &option_index)) != EOF) {
 		switch (optchar) {
                 case 0:
                 case 1:
@@ -1462,15 +1441,22 @@ main(int argc, char **argv)
                 case 'F':
                         Fflag++;
                         break;
+                case 'g':
+                        gflag++;
+                        break;
+                case 'h':
+			if (definition_header == NO_HEADER)
+				definition_header = AFTER_HEADER;
+                        break;
+                case 'I':
+                        Iflag++;
+                        break;
                 case 'm':
 			main_func = optarg;
                         break;
                 case 'n':
                         nflag++;
                         break;
-                case 'g':
-                        gflag++;
-                        break;
                 case 'o':
 			other_files = 1;
                         break;
@@ -1481,6 +1467,11 @@ main(int argc, char **argv)
 			Sflag++;
 			cgidir = optarg;
                         break;
+                case 'T':
+			table_flist = 1;
+			if (optarg && atoi(optarg) > 0)
+				flist_fields = atoi(optarg);
+                        break;
                 case 't':
 			title = optarg;
                         break;
@@ -1576,8 +1567,6 @@ main(int argc, char **argv)
 	}
 	if (dynamic && Sflag)
 		die("Current implementation doesn't allow both -D(--dynamic) and the -S(--secure-cgi).");
-	if (icon_list && !test("f", icon_list))
-		die("icon_list '%s' not found.", icon_list);
 	/*
 	 * decide directory in which we make hypertext.
 	 */
@@ -1717,38 +1706,24 @@ main(int argc, char **argv)
 	 * (0) make directories
 	 */
 	message("[%s] (0) making directories ...", now());
-	{
-		int mode = 0775;
-		char *dirs[] = {"files", "defines", SRCS, DEFS, REFS, INCS, INCREFS, SYMS};
-
-		int i, lim = sizeof(dirs) / sizeof(char *);
-
-		if (!test("d", distpath)) {
-			if (mkdir(distpath, 0777) < 0)
-				die("cannot make directory '%s'.", distpath);
-		}
-		for (i = 0; i < lim; i++) {
-			char *p = dirs[i];
-			/*
-			 * dynamic mode doesn't require these directories.
-			 */
-			if (dynamic)
-				if (!strcmp(p, DEFS) || !strcmp(p, REFS) || !strcmp(p, SYMS))
-					continue;
-			if (!symbol && !strcmp(p, SYMS))
-				continue;
-			path = makepath(distpath, p, NULL);
-			if (!test("d", path))
-				if (mkdir(path, mode))
-					die("cannot make '%s' directory.", p);
-		}
-		if (cgi && (fflag || cflag || dynamic)) {
-			path = makepath(distpath, "cgi-bin", NULL);
-			if (!test("d", path))
-				if (mkdir(path, mode) < 0)
-					die("cannot make cgi-bin directory.");
-		}
+	if (!test("d", distpath))
+		if (mkdir(distpath, 0777) < 0)
+			die("cannot make directory '%s'.", distpath);
+	make_directory_in_distpath("files");
+	make_directory_in_distpath("defines");
+	make_directory_in_distpath(SRCS);
+	make_directory_in_distpath(INCS);
+	make_directory_in_distpath(INCREFS);
+	if (!dynamic) {
+		make_directory_in_distpath(DEFS);
+		make_directory_in_distpath(REFS);
+		if (symbol)
+			make_directory_in_distpath(SYMS);
 	}
+	if (cgi && (fflag || cflag || dynamic))
+		make_directory_in_distpath("cgi-bin");
+	if (Iflag)
+		make_directory_in_distpath("icons");
 	/*
 	 * (1) make CGI program
 	 */
@@ -1832,7 +1807,7 @@ main(int argc, char **argv)
 		end_time = time(NULL);
 		message("Total %d files.", file_total);
 		T_makefileindex = end_time - start_time;
-		file_count += file_total;
+		html_count += file_total;
 		/*
 		 * [#] make include file index.
 		 */
@@ -1910,11 +1885,17 @@ main(int argc, char **argv)
 		}
 		message(" Good luck!\n");
 	}
-	/* This is not supported. */
-	if (icon_list && test("f", icon_list)) {
-		char command[MAXFILLEN];
-		snprintf(command, sizeof(command), "tar xzf %s -C %s", icon_list, distpath);
-		system(command);
+	if (Iflag) {
+		char src[MAXPATHLEN], dst[MAXPATHLEN];
+		int i, count = sizeof(icon_files) / sizeof(char *);
+
+		for (i = 0; i < count; i++) {
+			snprintf(src, sizeof(src), "%s/gtags/icons/%s.%s", datadir, icon_files[i], icon_suffix);
+			snprintf(dst, sizeof(dst), "%s/icons/%s.%s", distpath, icon_files[i], icon_suffix);
+			if (!test("f", src))
+				die("Icon file '%s' not found.", src);
+			copyfile(src, dst);
+		}
 	}
 	/*
 	 * Print statistics information.
