@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003 Tama Communications Corporation
+ * Copyright (c) 2003, 2005 Tama Communications Corporation
  *
  * This file is part of GNU GLOBAL.
  *
@@ -37,6 +37,8 @@
 #include "env.h"
 #include "strbuf.h"
 
+extern char **environ;
+
 /*
  * set_env: put environment variable.
  *
@@ -44,9 +46,7 @@
  *	i)	val	value
  */
 void
-set_env(var, val)
-	const char *var;
-	const char *val;
+set_env(const char *var, const char *val)
 {
 #ifdef HAVE_PUTENV
 	STRBUF *sb = strbuf_open(0);
@@ -72,3 +72,18 @@ get_home_directory(void)
 	return getenv("HOME");
 #endif
 }
+
+/*
+ * env_size: calculate the size of area used by environment.
+ */
+int
+env_size(void)
+{
+	char **e;
+	int size = 0;
+
+	for (e = environ; *e != NULL; e++)
+		size += strlen(*e) + 1;
+
+	return size;
+}
