@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 1998, 1999, 2000, 2002, 2005
+ * Copyright (c) 1997, 1998, 1999, 2000, 2002, 2005, 2006
  *	Tama Communications Corporation
  *
  * This file is part of GNU GLOBAL.
@@ -15,8 +15,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
  */
 
 #ifndef _STRBUF_H
@@ -28,10 +28,19 @@
 #else
 #include <strings.h>
 #endif
-#ifdef HAVE_STDARG_H 
 #include <stdarg.h>
-#else
-#include <varargs.h>
+
+#ifndef __attribute__
+/* This feature is available in gcc versions 2.5 and later.  */
+# if __GNUC__ < 2 || (__GNUC__ == 2 && __GNUC_MINOR__ < 5) || __STRICT_ANSI__
+#  define __attribute__(x)
+# endif
+/* The __-protected variants of `format' and `printf' attributes
+   are accepted by gcc versions 2.6.4 (effectively 2.7) and later.  */
+# if __GNUC__ < 2 || (__GNUC__ == 2 && __GNUC_MINOR__ < 7)
+#  define __format__ format
+#  define __printf__ printf
+# endif
 #endif
 
 /* #define STRBUF_LINK */
@@ -88,11 +97,6 @@ typedef struct _strbuf {
 	}\
 } while (0)
 
-#define strbuf_reset(sb) do {\
-	sb->curp = sb->sbuf;\
-	sb->alloc_failed = 0;\
-} while (0)
-
 #define strbuf_puts0(sb, s) do {\
 	strbuf_puts(sb, s);\
 	strbuf_putc(sb, '\0');\
@@ -115,8 +119,10 @@ void strbuf_dump(char *);
 #endif
 void __strbuf_expandbuf(STRBUF *, int);
 STRBUF *strbuf_open(int);
+void strbuf_reset(STRBUF *);
 void strbuf_clear(STRBUF *);
 void strbuf_nputs(STRBUF *, const char *, int);
+void strbuf_nputc(STRBUF *, int, int);
 void strbuf_puts(STRBUF *, const char *);
 void strbuf_puts_nl(STRBUF *, const char *);
 void strbuf_putn(STRBUF *, int);
@@ -125,17 +131,14 @@ char *strbuf_value(STRBUF *);
 void strbuf_trim(STRBUF *);
 void strbuf_close(STRBUF *);
 char *strbuf_fgets(STRBUF *, FILE *, int);
-#ifdef HAVE_STDARG_H
-void strbuf_sprintf(STRBUF *sb, const char *s, ...);
-#else
-void strbuf_sprintf();
-#endif
-STRBUF *strbuf_open_tempbuf();
+void strbuf_sprintf(STRBUF *sb, const char *s, ...)
+	__attribute__ ((__format__ (__printf__, 2, 3)));
+STRBUF *strbuf_open_tempbuf(void);
 void strbuf_release_tempbuf(STRBUF *);
 #ifdef STRBUF_LINK
 void strbuf_setname(STRBUF *, const char *);
 STRBUF *strbuf_getbuf(const char *);
-void strbuf_closeall();
+void strbuf_closeall(void);
 #endif
 
 #endif /* ! _STRBUF_H */
