@@ -87,8 +87,8 @@ done
 )
 
 echo "- Collecting reference manuals ..."
-commands="global gtags htags gtags-parser gozilla";
-perl ./convert.pl --menu $commands > doc/reference.txi
+commands="global gtags htags gtags-parser gozilla gtags-cscope";
+perl ./convert.pl --menu $commands > doc/reference.texi
 for d in `echo $commands`; do
 	perl ./convert.pl --info $d/manual.in > doc/$d.ref
 	echo "+ doc/$d.ref"
